@@ -23,6 +23,29 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+"""
+Face Classification objects, embedding pretrained DNN models,
+are implemented in module :mod:`inaFaceAnalyzer.face_classifier`.
+
+Face Classification classes inherits from abstract class :class:`FaceClassifier`.
+They assume
+
+They implement implement a :meth:`FaceClassifier.preprocessed_img_list` method,
+allowing.
+
+:meth:`FaceClassifier.__call__` method.
+
+
+>>> from inaFaceAnalyzer.face_classifier import Resnet50FairFaceGRA
+>>> classif = Resnet50FairFaceGRA()
+>>> classif.preprocessed_img_list(['./media/diallo224.jpg', './media/knuth224.jpg'])
+                filename  sex_decfunc  age_decfunc sex_label  age_label
+0  ./media/diallo224.jpg    -5.632371     3.072337         f  25.723367
+1   ./media/knuth224.jpg     7.255364     6.689072         m  61.890717
+
+sdfsdf
+"""
+
 import numpy as np
 import pandas as pd
 import numbers
@@ -92,34 +115,25 @@ class FaceClassifier(ABC):
     # TODO : Keras trick for async READ ?
     # bench execution time : time spent in read/exce . CPU vs GPU
     # TODO: add progress bar with verbose option
-    def preprocessed_img_list(self, lfiles, return_features=False, batch_len=32):
+    def preprocessed_img_list(self, lfiles, batch_len=32):
         """
         Performs classification on a list of preprocessed face images
         Preprocessed face images are assumed to contain a single face which is
         already detected, cropped, aligned and scaled to classifier's input
         dimensions (for now: 224*224 pixels)
 
-        Parameters
-        ----------
-        lfiles : list
-            list of image paths
-        return_features : bool, optional
-            Not yet implemented. The default is False.
-        batch_len : input, optional
-            Larger batch sizes results in faster processing times.
-            Batch lenght is dependent on available GPU memory
-            The default is 32 (suitable for a laptop GPU).
+        Args:
+            lfiles (list): list of image paths: ['/path/to/img1', '/path/to/img2']
+            batch_len (int, optional): DNN batch size. Larger batch_len results
+                in faster processing times.
+                Batch lenght is dependent on available GPU memory.
+                Defaults to 32 (suitable for a laptop GPU).
 
-        Returns
-        -------
-        df : pandas.DataFrame
-            a pandas dataframe with one record for each input image
+        Returns:
+            pandas.DataFrame. a DataFrame with one record for each input image
         """
         assert len(lfiles) > 0
 
-        if return_features:
-            raise NotImplementedError()
-
         lbatchret = []
 
         for i in range(0, len(lfiles), batch_len):
@@ -131,7 +145,7 @@ class FaceClassifier(ABC):
         df.insert(0, 'filename', lfiles)
         return df
 
-    def __call__(self, limg, return_features, verbose=False):
+    def __call__(self, limg, verbose=False):
         """
         Classify a list of images
         images are supposed to be preprocessed faces: aligned, cropped
@@ -156,7 +170,7 @@ class FaceClassifier(ABC):
             limg = [limg]
 
         assert np.all([e.shape == self.input_shape for e in limg])
-        batch_ret_feats, batch_ret_preds = self.inference(self.list2batch(limg))
+        batch_ret_preds = self.inference(self.list2batch(limg))
         batch_ret_preds = self.decisionfunction2labels(batch_ret_preds)
 
         if verbose:
@@ -165,29 +179,25 @@ class FaceClassifier(ABC):
                 print('prediction', pred)
 
         if islist:
-            if return_features:
-                return batch_ret_feats, batch_ret_preds
             return batch_ret_preds
 
         ret = next(batch_ret_preds.itertuples(index=False, name='FaceClassifierResult'))
-        if return_features:
-            return  batch_ret_feats[0, :], ret
         return ret
 
 class Resnet50FairFace(FaceClassifier):
 
     def __init__(self):
         m = keras.models.load_model(get_remote('keras_resnet50_fairface.h5'), compile=False)
-        self.model = tensorflow.keras.Model(inputs=m.inputs, outputs=m.outputs + [m.layers[-3].output])
+        self.model = tensorflow.keras.Model(inputs=m.inputs, outputs=m.outputs)
 
     def list2batch(self, limg):
         x = np.concatenate([np.expand_dims(img_to_array(e), axis=0) for e in limg])
         return tensorflow.keras.applications.resnet50.preprocess_input(x)
 
     def inference(self, x):
-        decisions, feats = self.model.predict(x)
+        decisions = self.model.predict(x)
         df = pd.DataFrame(decisions.ravel(), columns=['sex_decfunc'])
-        return feats, df
+        return df
 
     def decisionfunction2labels(self, df):
         df['sex_label'] = df.sex_decfunc.map(lambda x: 'm' if x > 0 else 'f' )
@@ -216,14 +226,23 @@ def _fairface_agedec2age(age_dec):
 
 
 class Resnet50FairFaceGRA(Resnet50FairFace):
+    """
+    Resnet50FairFaceGRA predicts age and gender and is the most accurate proposed.
+    It uses Resnet50 architecture and is trained to predict gender, age and race on FairFace.
+    After consultation of French CNIL (French data protection authority) and
+    DDD (French Rights Defender), racial classification layers were erased
+    from this public distribution in order to prevent their use for non ethical purposes.
+    These models can however be provided for free after examination of each demand.
+    """
+
     def __init__(self):
         m = keras.models.load_model(get_remote('keras_resnet50_fairface_GRA.h5'), compile=False)
-        self.model = tensorflow.keras.Model(inputs=m.inputs, outputs=m.outputs + [m.layers[-5].output])
+        self.model = tensorflow.keras.Model(inputs=m.inputs, outputs=m.outputs)
 
     def inference(self, x):
-        gender, _, age, feats = self.model.predict(x)
+        gender, _, age = self.model.predict(x)
         df = pd.DataFrame(zip(gender.ravel(), age.ravel()), columns=['sex_decfunc', 'age_decfunc'])
-        return feats, df
+        return df
 
     def decisionfunction2labels(self, df):
         df = super().decisionfunction2labels(df)
@@ -267,7 +286,7 @@ class OxfordVggFace(FaceClassifier):
         return self.vgg_feature_extractor(x)
 
     def inference(self, x):
-        return np.array(x), pd.DataFrame(self.gender_svm.decision_function(x), columns=['sex_decfunc'])
+        return pd.DataFrame(self.gender_svm.decision_function(x), columns=['sex_decfunc'])
 
 class Vggface_LSVM_YTF(OxfordVggFace):
     def __init__(self):
