@@ -23,6 +23,27 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+"""
+inaFaceAnalyzer module implements media analyzer classes allowing to process
+video or image streams.
+
+Analysis classes inherits from abstract class FaceAnalyzer.
+Custom face detection,  face classifier, eye detection and image preprocessing strategies can be provided in the constructor.
+
+>>> from inaFaceAnalyzer.inaFaceAnalyzer import VideoAnalyzer
+>>> # a video analyzer instance with default parameters
+>>> va = VideoAnalyzer()
+
+Analysis classes are designed as `*functions objects* or *functors* <https://en.wikipedia.org/wiki/Function_object>`_ :
+instances of these objects can be used as functions, executing the code
+implemented in `__call__` methods.
+__call__ method first argument is the path to the media to analyze.
+It returns a pandas DataFrame.
+
+>>> df = va(sample_vid)
+"""
+
+
 import pandas as pd
 from abc import ABC, abstractmethod
 from .opencv_utils import video_iterator, image_iterator, analysisFPS2subsamp_coeff
@@ -52,11 +73,12 @@ class FaceAnalyzer(ABC):
     def __init__(self, face_detector = None, face_classifier = None, batch_len=32, verbose = False):
         """
         Constructor
+
         Parameters
         ----------
-        face_detector : instance of face_detector.FaceDetector or None
+        face_detector : instance of :class:`inaFaceAnalyzer.face_detector.FaceDetector` or None
             if None, LibFaceDetection is used by default
-        face_classifier: instance of face_classifier.FaceClassifier or None
+        face_classifier: instance of :class:`inaFaceAnalyzer.face_classifier.FaceClassifier` or None
             if None, Resnet50FairFaceGRA is used by default (gender & age)
         verbose : boolean
             If True, will display several usefull intermediate images and results
@@ -150,13 +172,13 @@ class FaceAnalyzer(ABC):
 
             # if enough faces were found, process a batch of faces
             while len(lbatch_img) > self.batch_len:
-                df = self.classifier(lbatch_img[:self.batch_len], False, verbose=self.verbose)
+                df = self.classifier(lbatch_img[:self.batch_len], verbose=self.verbose)
                 ldf.append(df)
                 lbatch_img = lbatch_img[self.batch_len:]
 
 
         if len(lbatch_img) > 0:
-            df = self.classifier(lbatch_img, False, verbose=self.verbose)
+            df = self.classifier(lbatch_img, verbose=self.verbose)
             ldf.append(df)
 
         if len(ldf) == 0:
